package CsocketS;

import java.net.Socket;
import java.io.*;
import IS.hi.logir.cryptonite.Key;
import IS.hi.logir.cryptonite.KeyPair;
import CsocketS.*;

/**
 * Corresponde ao lado do Servidor da implementao das classes.
 * Recebe as mensagens atravs das interfaces comuns com o cliente, e coordena
 * as operaes no lado do servidor.
 * <p>
 * 
 * @author Luiz Angelo Barchet Estefanel
 * @author <a href="mailto:lae@inf.ufsm.br">lae@inf.ufsm.br</a>
 * @version %I%, %G%
 * @since JDK1.1
 */ 

public class CSServer_impl extends _CSServerImplBase
{
   private int id;

   private Socket socket;
   private BufferedInputStream inputs;
   private BufferedOutputStream outs;  

   private Packer PKG;

   private org.omg.CORBA.ORB orb_;

/**
 * Construtor da classe, inicializa o contato com o ORB e dispara a gerao
 * das chaves e <i>tickets</i>.
 *
 * @param orb     referncia ao ORB do Servidor, repassado pelo FactoryServer
 * @param id_	  nmero identificador de ordem do Servidor. til para contar
 *                quantos servidores j foram ativados.
 */
   public CSServer_impl(org.omg.CORBA.ORB orb,int id_) {
      orb_ = orb;
      id = id_;
      PKG = new Packer ();
   }


/**
 * Retorna o nmero identificador desse objeto servidor.
 *
 * @return inteiro que indica o nmero de ordem desse objeto.
 */
   public int get_id() {
      return id;
   }

/**
 * Requisita a eliminao desse objeto servidor, quando no  mais til (a 
 * conexo de rede j foi fechada).
 */
   public void destroy() {
      orb_.disconnect(this);
   }

/**
 * Fecha a conexo de rede com o servio.
 */
   public void close () {
    try { 
     if (socket == null)
       return;
     
     socket.close();
     socket = null;
     inputs = null;
     outs = null;
     PKG = null;
    } catch (IOException ex) {
    }
    this.destroy(); 
   }


/**
 * Cria uma conexo <i>socket</i> com a mquina provedora do servio.
 *
 * @param host   nome da mquina para qual deve ser aberta a conexo.
 * @param port   nmero indicador da porta de conexo ao servio.
 */
   public void open (String host, int port){
    try {
     close();
 
     socket = new Socket(host,port);
     inputs = new BufferedInputStream(socket.getInputStream());
     outs = new BufferedOutputStream(socket.getOutputStream());
    } catch (IOException ex) {
     System.out.println("CSServer_impl:open:nao conseguiu conectar ao servidor " + host);
	}
   }
 /**
  * Recebe do cliente um <i>buffer</i> (larrayHolder) para ser preenchido com os dados recebidos do servio.
  * @param rarray - Objeto do tipo larrayHolder que contm um <i>buffer</i>.
  * @return rarray - Retorna atravs de referncia
  */ 
   
  public int read (CsocketS.CSServerPackage.larrayHolder rarray)
  {
     int n;
     byte[] b = PKG.unpacking(rarray.value);
	try {
           n = inputs.read(b);
        } catch (IOException ex) {
            System.out.println ("CSServer_impl:read:deu pau...");
            return -1;
         }
        rarray.value = PKG.packing (b);
	return n;
   
   }

/**
 * Envia, atravs da conexo <i>socket</i> com o servio os dados recebidos do 
 * cliente.
 * 
 * @param warray  <i>array</i> de <i>bytes</i> que contm a mensagem segura.
 */
   public void write (byte[]  warray) {
	byte[] b = PKG.unpacking(warray);
        if (b == null)
 	  close();
	try {
           outs.write(b);
           outs.flush();
        } catch (IOException ex) {
            System.out.println("CSServer_impl:write:cannow write");
        }

   }


/**
 * Interface para troca de chaves entre o cliente e o servidor.
 * A troca d-se utilizando a estrutura rarray para receber a chave do cliente
 * e enviar a chave do servidor.
 *
 * @param rarray - Objeto larrayHolder que contm a chave do cliente na forma de <i>array</i> de <i>bytes</i>. Pela mesma estrutura 
 *               retorna a chave do servidor.
 */
   public void getKey(CsocketS.CSServerPackage.larrayHolder rarray) {
      while (PKG.get_Public() == null) {
         try {
            Thread.sleep(2000);
         } catch (InterruptedException ex) {
         }
      }
      try {
         ByteArrayInputStream bin = new ByteArrayInputStream(rarray.value);
         ObjectInputStream in = new ObjectInputStream(bin);
         Key key2 = (Key)in.readObject();
         PKG.set_Other(key2);
      } catch (Exception e) {
         e.printStackTrace(System.out);
      }
      try {
         ByteArrayOutputStream bout = new ByteArrayOutputStream();
         ObjectOutputStream out = new ObjectOutputStream(bout);
         out.writeObject(PKG.get_Public());
         byte arraya[] = bout.toByteArray();
         rarray.value = arraya;
      } catch (Exception e) {
         e.printStackTrace(System.out);
      }
   }


/**
 * Retorna o nmero de bytes disponveis para leitura no buffer do servio.
 *
 * @return nmero de bytes no buffer de envio do servio.
 */
   public int available() {
      try {
         return inputs.available();
      } catch (IOException ex) {
         return -1;
      }
   }
}
