package CsocketS;

import IS.hi.logir.cryptonite.*;

import java.util.*;
import java.math.*;
import java.io.*;

/**
 * Encryptool  a classe que realiza as operaes de cifragem e decifragem 
 * usadas nos processos de criptografia e assinatura digital.
 * <p>
 * 
 * @author Luiz Angelo Barchet Estefanel
 * @author <a href="mailto:lae@inf.ufsm.br">lae@inf.ufsm.br</a>
 * @version %I%, %G%
 * @since JDK1.1
 */ 
public class Encryptool {

   String keyType = "RSA";
   int keySize = 256;
   int mailLength = 25;
   int nameLength = 49; 
/**
 * Gera um par de chaves criptogrficas (RSA) com um comprimento de 256 bits.
 * O nome e o e-mail do proprietrio tambm so gerados aleatoriamente, buscando
 * aumentar segurana das chaves.
 * 
 * @return <b>KeyPair</b> - uma estrutura de dados definida no pacote
 *         criptogrfico <i>Cryptonite</i>.
 */
   public KeyPair make_key() {

      KeyPair keyPair = null;

      Random rd = new Random();
      byte nome[] = new byte[nameLength];
      rd.nextBytes(nome);
      String NomeS = new String(nome);

      byte mail[] = new byte[mailLength];
      rd.nextBytes(mail);
      String MailS = new String(mail);
      try {
         //keyPair = Key.create(keyType, keySize, "James Bond", "jbond@mi5.uk");
         keyPair = Key.create(keyType,keySize, NomeS, MailS);
	 return keyPair;
      } catch (CryptoException e) {
         System.err.println(e);
         return null;
      }
   }

/**
 * Mtodo de cifragem dos dados. Utiliza os mtodos de cifragem estabelecidos
 * no pacote criptogrfico <i>Cryptonite</i>.
 * Pode ser utilizada para criptografia, ao fornecer uma chave pblica, ou 
 * para a assinatura digital, fornecendo uma chave privada.
 *
 * @param <b>message</b> - Mensagem a ser cifrada.
 * @param <b>chave</b> - Chave privada ou pblica para efetuar uma assinatura ou
 *                cifragem, respectivamente.
 *
 * @return <b><i>array</i> de <i>bytes</i> - A mensagem cifrada, aps o processo.
 */
   public byte[] encrypt(byte[] message, Key chave) {

	byte dest[] = chave.encrypt(message);	
	return dest;

   }

/**
 * Mtodo de decifragem dos dados. Utiliza os mtodos de cifragem estabelecidos
 * no pacote criptogrfico <i>Cryptonite</i>.
 * Pode ser utilizada para decriptografia, ao fornecer uma chave privada, ou
 * para a verificao de uma assinatura digital, fornecendo uma chave pblica.
 *
 * @param <b>message</b> - Mensagem a ser decifrada.
 * @param <b>chave</b> - Chave pblica ou privada para verificar uma assinatura ou
 *                decifrar uma mensagem, respectivamente.
 *
 * @return <b><i>array</i> de <i>bytes</i></b> - A mensagem decifrada, aps o processo.
 */

   public byte[] decrypt(byte[] message, Key chave) {
   
	byte dest[] = chave.decrypt(message);
        return dest;
   
   }
}
      
