package CsocketS;

import IS.hi.logir.cryptonite.*;
import java.util.Vector;
import CsocketS.*;


/**
 * Classe que coordena as operaes referentes  adaptao da mensagem para
 * uma estrutura segura para a transmisso. Coordena tambm a extrao da 
 * mensagem cifrada recebida. Essas operaes so:
 * <ul>
 * <li>Gerao das chaves criptogrficas;
 * <li>Troca de chaves pblicas entre o cliente e o servidor;
 * <li>Inicializao e verificao dos <i>tickets</i>;
 * <li>Serializao e desserializao do pacote de dados (mensagem + <i>tickets</i>);
 * <li>Assinatura e verificao da origem da mensagem;
 * <li>Cifragem e decifragem da mensagem transmitida.
 * </ul>
 *
 * @author Luiz Angelo Barchet Estefanel
 * @author <a href="mailto:lae@inf.ufsm.br">lae@inf.ufsm.br</a>
 * @version %I%, %G%
 * @since JDJ1.0
 */

public class Packer {

   private KeyPair myKeys = null;
   private Key otherKey = null;
   private Vector otherSideTicket = null;

   private TicketUtil TUTIL = new TicketUtil(); 
   private Encryptool ECT = new Encryptool();
   private Serializador SLR = new Serializador();

/**
 * Construtor da classe Packer, inicializa alguns parmetros (<i>tickets</i>,
 * chaves criptogrficas).
 */
   Packer () {

      this.myKeys = ECT.make_key();
      otherSideTicket = new Vector();
      String osts = "0";
      long ostl = 0;
      Long ostL = new Long (ostl);
      otherSideTicket.addElement(osts);
      otherSideTicket.addElement(ostL);
   }

/**
 * Extrai a mensagem a partir dos dados seguros recebidos via rede.
 * Executa as seguintes operaes, em ordem:
 * <ul>
 * <li>decifragem dos dados, com a chave privada;
 * <li>verificao da assinatura dos dados, com a chave pblica do emissor;
 * <li>desserializao do pacote de dados;
 * <li>verificao dos <i>tickets</i>;
 * <li>extrao da mensagem;
 * </ul>
 *
 * @param message   <b><i>array</i> de <i>bytes</i></b> recebido pela rede, contendo
 *                  a mensagem cifrada e assinada.
 *
 * @return    <b><i>array</i> de <i>bytes</i></b> contendo a mensagem original extrada
 *            da mensagem segura que veio pela rede.
 */

   public byte[] unpacking (byte[] message) {
         byte decriptado[] = ECT.decrypt(message,myKeys.getPrivate());
         byte desassinado[] = ECT.decrypt(decriptado,otherKey);
         Vector vet1 = SLR.desserializar(desassinado);
         if (TUTIL.testTicket((Vector)vet1.elementAt(1))) {
            otherSideTicket = (Vector)(vet1.elementAt(0));
            byte msg[] = ((String)vet1.elementAt(2)).getBytes();	
            return msg;
         }
         else {
            return null;
         }
   }

/**
 * Monta uma mensagem segura, apta a trafegar na rede.
 * Executa as seguintes operaes, em ordem:
 * <ul>
 * <li>gerao de um novo <i>ticket</i> identificador dessa transmisso;
 * <li>criao de um pacote de dados contendo a mensagem e os <i>tickets</i>;
 * <li>serializao do pacote de dados;
 * <li>assinatura dos dados, com a chave privada do emissor;
 * <li>cifragem dos dados, com a chave pblica do receptor;
 * </ul>
 *
 * @param message   <b><i>array</i> de <i>bytes</i></b> contendo a mensagem original. 
 *
 * @return    <b><i>array</i> de <i>bytes</i></b> contendo a mensagem segura, apta ao
 *            trfego na rede.
 */
   public byte[] packing (byte[] message) {
	 String msg = new String(message);
         TUTIL.makeTicket();
         Vector Retorno = new Vector(3);
         Retorno.addElement(TUTIL.getTicket());
         Retorno.addElement(otherSideTicket);
         Retorno.addElement(msg); 
         byte volta[] = SLR.serializar(Retorno);
         byte assinado[] = ECT.encrypt(volta,myKeys.getPrivate());
         byte encriptado[] = ECT.encrypt(assinado,otherKey);
         return encriptado;
   }

/**
 * Mtodo utilizado na troca das chaves, entre os processos cliente e servidor.
 *
 * @return <b>Key</b> - chave pblica do objeto.
 */
   public Key get_Public() {
      return  myKeys.getPublic();
   }

/**
 * Mtodo que armazena a chave pblica da outra parte, trocada pela rede.
 *
 * @param <b>other</b> -  Chave pblica enviada pelo outro participante da conexo.
 */  
   public void set_Other(Key other) {
      this.otherKey = other;
   }
} 
