package CsocketS;

import java.util.Random;
import java.util.Vector;

/**
 * Classe responsvel pela gerao dos <i>tickets</i> utilizados para a 
 * identificao das conexes e manuteno do <i>timeout</i>.
 * Realiza tanto a criao de novos <i>tickets</i> quanto a verificao da
 * validade dos <i>tickets</i> recebidos.
 *
 * @author Luiz Angelo Barchet Estefanel
 * @author <a href="mailto:lae@inf.ufsm.br">lae@inf.ufsm.br</a>
 * @version %I%, %G%
 * @since JDJ1.0
 */
public class TicketUtil {

   private Vector myTicket=null;
   public int TicketNumLength = 30;

/** 
 * Construtor do Objeto TicketUtil.
 * Inicializa com a criao de um <i>ticket</i> padro, para fim de conexo
 * entre as partes.
 */
   TicketUtil() {

      myTicket = new Vector();
      String tsts = "0";
      long tstl = 0;
      Long tstL = new Long (tstl);
      myTicket.addElement(tsts);
      myTicket.addElement(tstL);

   }

/**
 * Gera um novo <i>ticket</i>, com um identificador randmico e um <i>timestamp</i>
 * correspondente ao "tempo" local.
 * Esse <i>ticket</i> no  retornado porque deve ser usado tanto na transmisso
 * de um novo <i>ticket</i> quanto na verificao do <i>ticket</i> recebido.
 */
   public void makeTicket() {

      myTicket = new Vector();
      long tstl = System.currentTimeMillis();
      Random randy = new Random(tstl);
      byte dropping[] = new byte[TicketNumLength];
      randy.nextBytes(dropping);
      String tsts = new String (dropping);
      Long tstL = new Long(tstl);

      myTicket.addElement(tsts);
      myTicket.addElement(tstL);

   }
   
/**
 * Mtodo para buscar o <i>ticket</i> atual.
 *
 * @return  <b><i>ticket</i></b> - <i>Ticket</i> atualmente ativo no sistema.
 */ 
   public Vector getTicket () {
      return myTicket;
   }


/**
 * Verificao do <i>ticket</i> recebido, que corresponde ao </i>ticket</i>
 * gerado por esse objeto, enviado na transmisso anterior.
 * 
 * Compara o nmero identificador nico do <i>ticket</i>. Se for semelhante,
 * verifica se o <i>ticket</i> no expirou o tempo limite (<i>timeout</i>).
 *
 * @param ticket  O <i>ticket</i> correspondente ao receptor, armazenado junto 
 *                com a mensagem recebida.
 * 
 * @return        <b>true</b> se o <i>ticket</i> recebido confere com o
 *                <i>ticket</i> armazenado, e ainda no expirou o tempo mximo;
 *                <p><b>false</b> se o <i>ticket</i> no confere ou j
 *                ultrapassou o prazo limite de tempo.
 */ 
   public boolean testTicket (Vector ticket) {

      String tsts = (String)myTicket.elementAt(0);
      Long tstL = (Long)myTicket.elementAt(1);

      String ots = (String)ticket.elementAt(0);
      Long otL = (Long)ticket.elementAt(1); 

      long difference = 0;

      if (tsts.compareTo(ots) == 0) {
         if (tstL.longValue() == otL.longValue()) {
	    if (tstL.longValue() == 0)
               difference = tstL.longValue();
            else 
               difference = System.currentTimeMillis() - otL.longValue();
            if (difference < 900000) {
               return true;
            }
            else return false;
         }
         else return false;
      }
      else return false;
   }

}
