package IS.hi.logir.cryptonite.test;

import IS.hi.logir.cryptonite.*;

import java.util.*;
import java.math.*;
import java.io.*;

/**
 * An application used to test and time encryption and decryption.
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a> (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 */
public class TestEncryption {
    
    static Date lastCallToTimer;
    
    /**
     * Write the number of minutes since the last call to timer
     * to the stream out. */
    public static void timer(PrintStream out){
        Date now= new Date();
        if (lastCallToTimer!=null){
            long ms=(now.getTime()-lastCallToTimer.getTime());
            if (out!=null)
                out.println((ms/1000) + "." + (ms%1000) + "s");
        }
        lastCallToTimer = now;
    }	

    // Print a message if the trwo arrays of bytes are not identical
    public static boolean checkDiff(Data A, Data B){
        if (A.n != B.n){
	    System.err.println("ERROR: Different lengths");
	    return false;
        } else {
            for (int i=0; i<A.n; i++)
                if (A.buf[i]!=B.buf[i]){
		    System.err.println("ERROR: Difference at offset "+i);
                    return false;
		}
	}
	return true;
    }
    
    private static void error(){
	System.err.println("Use: java TestEncryption [+|-] [type size]");
	System.err.println("  +/-  specifies whether to use session keys.");
	System.err.println("  type is the name of the key type to use. F.ex. RSA, Tea, Caesar");
	System.err.println("  size is the size of key to create. The meaning varies with type");
    }
    
    /**
     * main entrypoint - starts the application
     *
     * @exception IOException if there is any kind of I/O or encryption error.
     * @exception ClassNotFoundException if there is trouble with the
     *            deserialization of the KeyRing.
     *
     * @param args java.lang.String[]
     */ 
    public static void main(java.lang.String[] arg) throws IOException, ClassNotFoundException {
        timer(null);
        
	String keyType = "RSA";
	int keySize = 256;
	boolean sessionKey = false;
	for (int i=0; i<arg.length; i++){
	    if (arg[i].equals("+"))
		sessionKey=true;
	    else if (arg[i].equals("-"))
		sessionKey=false;
	    else if (i==arg.length-1) {
		// no size parameter follows
		error();
		return;
	    } else {
		keyType = arg[i];
		keySize = Integer.parseInt(arg[++i]);
	    }
	}
	
        KeyRing keyRing = new KeyRing();
	KeyPair keyPair = null;
	try {
	    keyPair = Key.create(keyType, keySize, "James Bond", "jbond@mi5.uk");
	} catch (CryptoException e) {
	    System.err.println(e);
	    error();
	    return;
	}

        Key pub = keyPair.getPublic();
        Key pri = keyPair.getPrivate();
        keyRing.addTrustedPair(keyPair);
        
        Random rnd=new Random();

        System.err.println(pub);
        System.err.print("Time to generate key: ");
        timer(System.err);
        System.err.println("-----------------------------------");

	Data A=new Data(2*1024);
	rnd.nextBytes(A.buf);
	A.n=2048;
	System.err.println("Using buffer of "+A.n+" random bytes");
	timer(null);
	Data B = new Data();
	pub.encrypt(A,B);
	System.err.print("Time to encrypt: ");
	timer(System.err);
	Data C=new Data();
	pri.decrypt(B,C);
	System.err.print("Time to decrypt: ");
	timer(System.err);
	if (!checkDiff(A,C))
	    return;
  	System.err.println("-----------------------------------"); 

        EncryptStream EE=new EncryptStream(new FileOutputStream("encrypted"), pub);
	if (sessionKey){
	    System.err.println("Setting session key");
	    EE.setSessionKey(new TeaKey(32, "", ""));
	}
        PrintWriter E = new PrintWriter(EE);
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        System.err.println("Please enter some text on standard input: (or pipe something to the program)");
	timer(null);
        String S;
	while ((S=in.readLine())!=null)
            E.println(S);
        E.close();
        System.err.println("The entered text has now been encrypted and written to file.");
	System.err.print("Time: ");
	timer(System.err);
	
        ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("keyring"));
        oos.writeObject(keyRing);
        keyRing=null;
        pub=null;
        pri=null;
        System.err.println("The key-ring has been written to file.");
        oos.close();
        oos=null;

        ObjectInputStream ois = new ObjectInputStream(new FileInputStream("keyring"));
        keyRing = (KeyRing)ois.readObject();
        ois.close();
        System.err.println("The key-ring has been read back.");

  	System.err.println("-----------------------------------");
        System.err.println("The encrypted text will now be read and decrypted.");
	timer(null);
        BufferedReader D=new BufferedReader(new InputStreamReader(new DecryptStream(new FileInputStream("encrypted"),  null, keyRing)));
        while ((S=D.readLine())!=null)
            System.out.println(S);
        D.close();
	System.err.print("Time: ");
	timer(System.err);
    }
}
