package IS.hi.logir.cryptonite.util;

import IS.hi.logir.cryptonite.*;
import java.io.*;

/**
 * A utility program to manage keyring files. Still under construction.
 * 
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a> (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 */
public class ckey{
    
    static int NONE   = 0;
    static int VIEW   = 1;
    static int CREATE = 2;
    static int ADD    = 3;
    static int REMOVE = 4;

    static KeyRing keyRing;
    static boolean modified=false;
    
    public static void view(boolean pri){
	readRing();
	
	System.out.println("view " + (pri ? "private" : "public") + "keys");
	Key[] keys=null;
	if (pri)
	    keys = keyRing.findListPri(null,null);
	else
	    keys = keyRing.findListPub(null,null);
	for (int i=0; i<keys.length; i++)
	    System.out.println(keys[i]);
    }
    
    public static void create(String type, int size, String name, String mail) throws BadKeyClassException {
	readRing();

	System.out.println("Creating keys, please wait...");
	KeyPair keyPair = Key.create(type, size, name, mail);
	keyRing.addTrustedPair(keyPair);
	modified=true;
	System.out.println("Created the keys:");
	System.out.println(keyPair.getPublic());
	System.out.println(keyPair.getPrivate());	
    }
    
    public static void add(String fname){
	readRing();
	
	System.out.println("add");
    }
    
    public static void remove(String mask){
	readRing();
	
	System.out.println("remove");
    }
    
    public static void help(){
	System.out.println("Syntax:");
	System.out.println("  java ckey");
	System.out.println("  java ckey v [pri]");
	System.out.println("  java ckey c type size name mail");
	System.out.println("  java ckey a file");
	System.out.println("  java ckey r name:mail");
	System.out.println();
	System.out.println("Commands:");
	System.out.println("  v    View keys. This is the default");
	System.out.println("  c    Create keyof the given type and the given size, for\n"+
			   "       the entity called <name> with e-mail address <mail>.");
	System.out.println("  a    Add a key from <file> to the keyring.");
	System.out.println("  r    Remove keys with name fields containing <name>\n"+
			   "       and mail fields containing <mail>");
	System.out.println();
	System.out.println("Key types:");
	System.out.println("  RSA     Very secure, but fairly slow public key algorithm.\n"+
			   "          Normal keys will be of about 1024 bits.");
	System.out.println("  Tea     I don't really know this one very well... a good size is about 16.");
	System.out.println("  Caesar  Very insecure, don't use it. The size is ignored");
    }

    private static void readRing(){
	try {
	    ObjectInputStream ois = new ObjectInputStream(new FileInputStream("keyring"));
	    keyRing = (KeyRing)ois.readObject();
	    ois.close();
	} catch (Exception e){
	    System.err.println("Unable to read keyring. Do you wish to create an empty one? [y/n]");
	    try {
		int c=System.in.read();
		while(c!='y' && c!='Y' && c!='n' && c!='N')
		    c=System.in.read();
		if(c=='n' || c=='N' || c==-1)
		    return;
	    } catch (IOException ee) {
		System.err.println("Error reading from the keyboard.");
		System.exit(1);
	    }
	    keyRing = new KeyRing();
	}
    }
    
    public static void main(String[] arg) throws BadKeyClassException {
	if (arg.length==0)
	    view(false);
	else if (arg[0].equals("v") && arg.length==1)
	    view(false);
	else if (arg[0].equals("v") && arg.length==2 && arg[1].equals("pri"))
	    view(true);
	else if (arg[0].equals("c") && arg.length==5)
	    create(arg[1], Integer.parseInt(arg[2]), arg[3], arg[4]);
	else if (arg[0].equals("a") && arg.length==2)
	    add(arg[1]);
	else if (arg[0].equals("r") && arg.length==2)	
	    remove(arg[1]);
	else
	    help();
	
	if (modified) try {
	    ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("keyring"));
	    oos.writeObject(keyRing);
	    oos.close();
	} catch (IOException e) {
	    System.err.println("Unable to write modified keyring to file.");
	}
    }
    
}
