package CsocketS;

import org.omg.CORBA.*;
import java.net.*;
import java.io.*;
import CsocketS.*;
import CsocketS.CSServerPackage.*;
import IS.hi.logir.cryptonite.Key;

/**
 * Classe responsvel pela transmisso dos dados, no lado do cliente.
 * Contm mtodos com a interface semelhante s de uma conexo Socket tradicional. 
 *
 * @author Luiz Angelo Barchet Estefanel
 * @author <a href="mailto:lae@inf.ufsm.br">lae@inf.ufsm.br</a>
 * @version %I%, %G%
 * @since JDJ1.0
 */

public class CSClient {

   private CSServer CServer = null;
   private Packer PKG = null;
   private ORB orb = null;
   private ConnFactory conn_factory = null;


/**
 * Inicializa o ORB do lado do cliente, e conecta-se ao Servidor utilizando
 * o arquivo que contm a referncia IOR, disponibilizado pelo Servidor.
 * Requisita a criao de um processo servidor prprio, para a manuteno da
 * conexo.
 * Tambm inicia a gerao dos <i>tickets</i> e das chaves criptogrficas (objeto PKG)
 */
  
   public void connect() {

      String argsm[] = new String[4];
	argsm[0]=new String("CSClient");
	argsm[1] = new String ("-ORBthreaded");
	argsm[2] = new String ("-ORBtrace_level");
	argsm[3] = new String ("2");


      orb = org.omg.CORBA.ORB.init(argsm, new java.util.Properties());

        String location = "http://quenia.cpd.ufsm.br/~lae/CsocketS/CsocketS.ref";
        try {
          URL url = new URL(location);
          URLConnection conn = url.openConnection();
          BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
          String ref = in.readLine();
          in.close();
          org.omg.CORBA.Object object = orb.string_to_object(ref);

          try {
             conn_factory = ConnFactoryHelper.narrow(object);
             if (conn_factory == null) {
                System.err.println("No ConnFactory implementation enabled");
                System.exit(1);
             }
          }
          catch (org.omg.CORBA.SystemException se) {
             System.err.println("Creating Conn_Factory " + se);
          }

          try {
             CServer = conn_factory.CreateConn();
          }
          catch (org.omg.CORBA.SystemException se) {
             System.out.println("CServer " + se);
          }
          if (CServer != null) {
             PKG = new Packer();
             try {
                ByteArrayOutputStream bout = new ByteArrayOutputStream();
                ObjectOutputStream out = new ObjectOutputStream(bout);
                out.writeObject(PKG.get_Public());
                byte arraya[] = bout.toByteArray();
                larrayHolder ahold = new larrayHolder(arraya);
                CServer.getKey(ahold);
                try {
                   ByteArrayInputStream bin = new ByteArrayInputStream(ahold.value);
                   ObjectInputStream inO = new ObjectInputStream(bin);
                   Key key2 = (Key)inO.readObject();
                   PKG.set_Other(key2);
                } catch (Exception e) {
                e.printStackTrace(System.out);
                }
             } catch (Exception e) {
                e.printStackTrace(System.out);
             }
          }
        } catch (java.io.IOException ex) {
          System.out.println ("CSClient:CSClient:Cannot get IOR");
        }
   }


/**
 * Requisita ao processo servidor para encerrar a conexo com o <i>host</i>
 * provedor do servio.
 */
    public void close() throws IOException {
        CServer.close(); 
    }

/**
 * Inicia uma conexo ao <i>host</i> provedor do servio desejado.
 *
 * @param host   String que contm o nome do <i>host</i> a qual deseja-se conectar.
 * @param port   Int que indica a porta do servio desejado (por exemplo, 23 para <i>telnet</i>.
*/
    public void open(String host, int port) throws IOException {
       CServer.open(host,port);
   }


/**
 * Mtodo que recebe os dados encriptados, vindos do servidor.
 *
 * @param buf <i>array</i> de <i>bytes</i> que ser preenchido com os dados recebidos.
 *
 * @return  o nmero de <i>bytes</i>armazenados no buffer buf.
 */
   public int read(byte[] buf) throws IOException {
        int             n;
        byte[] buffy = PKG.packing(buf);
        larrayHolder ahold = new larrayHolder(buffy);
    	   n = CServer.read(ahold);
        byte[] b = PKG.unpacking(ahold.value);
        if (b == null)
 	  close();
	 for (int i = 0;i<b.length;++i){
	    buf[i] = b[i];
         }
         return n;
    }

/**
 * Mtodo para enviar os dados seguros pela rede, como no Socket padro.
 * 
 * @param buf  Buffer preenchido com os dados a serem enviados.
 */
    public void write(byte[] buf) {
        byte[] ahold = PKG.packing(buf);
           CServer.write(ahold);
    }


/**
 * Mtodo que retorna o nmero de <i>bytes</i> que ainda restam para a leitura,
 * no lado do servidor.
 *  utilizado por aplicaes como o <i>telnet</i>, que transmitem e recebem
 * os dados caracter a caracter. Em aplicaes como o <i>TN3270</i> os dados
 * sempre so transmitidos em grandes blocos (pgina a pgina), no havendo
 * a necessidade desse mtodo.
 *
 * @return o nmero de <i>bytes</i> disponveis para leitura no buffer do servidor.
 */
    public int available() {
       return CServer.available();
    }
}
